import React from 'react';
import ReactDOM from 'react-dom';
import { useState, useEffect } from 'react';
import { csv, scaleLinear, scaleOrdinal, extent } from 'd3';
import { useData } from './useData';
import { AxisBottom } from './AxisBottom';
import { AxisLeft } from './AxisLeft';
import { Marks } from './Marks';

const width = 960;
const height = 450;
const radius = 10;
const margin = { top: 10, bottom: 65, right: 200, left: 80 };

const options = [
    { value:'petal_length', label: 'Petal Length' },
    { value:'petal_width', label: 'Petal Width' },
    { value:'sepal_length', label: 'Sepal Length' },
    { value:'sepal_width', label: 'Sepal Width' }
  ];

const getLabel = (value) => {
	for(let i=0; i<options.length; i++) {
  	if (options[i].value === value) {
      return options[i].label;
    }
  }
}

const App = () => {
  const data = useData();
  
  const initialXValueSelected = 'petal_length';
  const [xValueSelected, setXValueSelected] = useState(initialXValueSelected);
  
  const initialYValueSelected = 'sepal_width';
  const [yValueSelected, setYValueSelected] = useState(initialYValueSelected);
  
  if (!data) {
    return <div>Loading...</div>;
  }
  
  const innerHeight = height - margin.top - margin.bottom;
  const innerWidth = width - margin.left - margin.right;
  
  const xValue = d => d[xValueSelected];
  const yValue = d => d[yValueSelected];
  
  const xScale = scaleLinear().domain(extent(data, xValue)).range([0, innerWidth]);
  const yScale = scaleLinear().domain(extent(data, yValue)).range([innerHeight, 0]);
  const colorScale = scaleOrdinal().domain(data.map(d => d.species)).range(['#42A5B3','#F2D5A7','#5E6BCA']);
  
  return (
    <>
      <div>
        <label for="x-option">X:</label>
        <select id="x-option" onChange={(e) => setXValueSelected(e.target.value)}>
          {
          	options.map(({value, label}) => <option value={value} selected={value === xValueSelected}>{label}</option>)
          }
        </select>
        <label for="y-option">Y:</label>
        <select id="y-option" onChange={(e) => setYValueSelected(e.target.value)}>
          {
          	options.map(({value, label}) => <option value={value} selected={value === yValueSelected}>{label}</option>)
          }
        </select>
      </div>
    <svg width={width} height={height}>
      <g transform={`translate(${margin.left},${margin.top})`}>
        <AxisBottom xScale={xScale} innerHeight={innerHeight} />
        <AxisLeft yScale={yScale} innerWidth={innerWidth} />
        <text
          className="axis-label"
          x={innerWidth / 2}
          style={{textAnchor:'middle'}}
          y={innerHeight + 35}
          >{getLabel(xValueSelected)}</text>
        <text
          className="axis-label"
          style={{textAnchor:'middle'}}
          transform={`translate(${-margin.left/2},${innerHeight/2}) rotate(-90)`}
          >{getLabel(yValueSelected)}</text>
      	<Marks data={data} xScale={xScale} yScale={yScale} xValue={xValue} yValue={yValue} radiusCircle={radius} colorScale={colorScale} />
      </g>
      <g transform={`translate(${innerWidth + 120}, ${margin.top + 30})`}>
        <text className="axis-label">Species</text>
        {
        	colorScale.domain().map((tickValue, i) => <g className="tick" transform={`translate(20,${(i+1)*30})`}>
                                  <circle r={radius} fill={colorScale(tickValue)} />
                                    <text
                                      x={2 * radius}
                                      dy="0.32em"
                                      >{tickValue}</text>
                                  </g>)
        }
      </g>
    </svg>
      </>
  );
  
}

ReactDOM.render(<App />, document.getElementById('root'));
